##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Params(base.ParamsModelBase):
    """Parameters model for :obj:`com.atproto.label.subscribeLabels`."""

    cursor: t.Optional[int] = None  #: The last known event seq number to backfill from.


class ParamsDict(te.TypedDict):
    cursor: te.NotRequired[t.Optional[int]]  #: The last known event seq number to backfill from.


class Labels(base.ModelBase):
    """Definition model for :obj:`com.atproto.label.subscribeLabels`."""

    labels: t.List['models.ComAtprotoLabelDefs.Label']  #: Labels.
    seq: int  #: Seq.

    py_type: te.Literal['com.atproto.label.subscribeLabels#labels'] = Field(
        default='com.atproto.label.subscribeLabels#labels', alias='$type', frozen=True
    )


class Info(base.ModelBase):
    """Definition model for :obj:`com.atproto.label.subscribeLabels`."""

    name: str  #: Name.
    message: t.Optional[str] = None  #: Message.

    py_type: te.Literal['com.atproto.label.subscribeLabels#info'] = Field(
        default='com.atproto.label.subscribeLabels#info', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class ListViewBasic(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    cid: str  #: Cid.
    name: str = Field(min_length=1, max_length=64)  #: Name.
    purpose: 'models.AppBskyGraphDefs.ListPurpose'  #: Purpose.
    uri: str  #: Uri.
    avatar: t.Optional[str] = None  #: Avatar.
    indexed_at: t.Optional[str] = None  #: Indexed at.
    viewer: t.Optional['models.AppBskyGraphDefs.ListViewerState'] = None  #: Viewer.

    py_type: te.Literal['app.bsky.graph.defs#listViewBasic'] = Field(
        default='app.bsky.graph.defs#listViewBasic', alias='$type', frozen=True
    )


class ListView(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    cid: str  #: Cid.
    creator: 'models.AppBskyActorDefs.ProfileView'  #: Creator.
    indexed_at: str  #: Indexed at.
    name: str = Field(min_length=1, max_length=64)  #: Name.
    purpose: 'models.AppBskyGraphDefs.ListPurpose'  #: Purpose.
    uri: str  #: Uri.
    avatar: t.Optional[str] = None  #: Avatar.
    description: t.Optional[str] = Field(default=None, max_length=3000)  #: Description.
    description_facets: t.Optional[t.List['models.AppBskyRichtextFacet.Main']] = None  #: Description facets.
    viewer: t.Optional['models.AppBskyGraphDefs.ListViewerState'] = None  #: Viewer.

    py_type: te.Literal['app.bsky.graph.defs#listView'] = Field(
        default='app.bsky.graph.defs#listView', alias='$type', frozen=True
    )


class ListItemView(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    subject: 'models.AppBskyActorDefs.ProfileView'  #: Subject.
    uri: str  #: Uri.

    py_type: te.Literal['app.bsky.graph.defs#listItemView'] = Field(
        default='app.bsky.graph.defs#listItemView', alias='$type', frozen=True
    )


ListPurpose = t.Union['models.AppBskyGraphDefs.Modlist', 'models.AppBskyGraphDefs.Curatelist']  #: List purpose

Modlist = te.Literal[
    'app.bsky.graph.defs#modlist'
]  #: A list of actors to apply an aggregate moderation action (mute/block) on.

Curatelist = te.Literal[
    'app.bsky.graph.defs#curatelist'
]  #: A list of actors used for curation purposes such as list feeds or interaction gating.


class ListViewerState(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`."""

    blocked: t.Optional[str] = None  #: Blocked.
    muted: t.Optional[bool] = None  #: Muted.

    py_type: te.Literal['app.bsky.graph.defs#listViewerState'] = Field(
        default='app.bsky.graph.defs#listViewerState', alias='$type', frozen=True
    )


class NotFoundActor(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`. indicates that a handle or DID could not be resolved."""

    actor: str  #: Actor.
    not_found: bool = Field(frozen=True)  #: Not found.

    py_type: te.Literal['app.bsky.graph.defs#notFoundActor'] = Field(
        default='app.bsky.graph.defs#notFoundActor', alias='$type', frozen=True
    )


class Relationship(base.ModelBase):
    """Definition model for :obj:`app.bsky.graph.defs`. lists the bi-directional graph relationships between one actor (not indicated in the object), and the target actors (the DID included in the object)."""

    did: str  #: Did.
    followed_by: t.Optional[
        str
    ] = None  #: if the actor is followed by this DID, contains the AT-URI of the follow record.
    following: t.Optional[str] = None  #: if the actor follows this DID, this is the AT-URI of the follow record.

    py_type: te.Literal['app.bsky.graph.defs#relationship'] = Field(
        default='app.bsky.graph.defs#relationship', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing_extensions as te
from pydantic import Field

from atproto_client.models import base


class Main(base.ModelBase):
    """Definition model for :obj:`com.atproto.repo.strongRef`."""

    cid: str  #: Cid.
    uri: str  #: Uri.

    py_type: te.Literal['com.atproto.repo.strongRef'] = Field(
        default='com.atproto.repo.strongRef', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.identity.getRecommendedDidCredentials`."""

    also_known_as: t.Optional[t.List[str]] = None  #: Also known as.
    rotation_keys: t.Optional[
        t.List[str]
    ] = None  #: Recommended rotation keys for PLC dids. Should be undefined (or ignored) for did:webs.
    services: t.Optional['UnknownType'] = None  #: Services.
    verification_methods: t.Optional['UnknownType'] = None  #: Verification methods.



class AtProtocolError(Exception):
    """Base exception."""


class InvalidNsidError(AtProtocolError):
    ...


class InvalidAtUriError(AtProtocolError):
    ...


class InvalidCARFile(AtProtocolError):
    ...


class DAGCBORDecodingError(AtProtocolError):
    ...



import typing as t
from abc import ABC, abstractmethod

if t.TYPE_CHECKING:
    from atproto_core.did_doc import DidDocument

    from atproto_identity.cache.models import CachedDidResult


_DEFAULT_STALE_TTL = 60 * 60  # 1 hour
_DEFAULT_MAX_TTL = 60 * 60 * 24  # 1 day


GetDocCallback = t.Callable[[], t.Optional['DidDocument']]
AsyncGetDocCallback = t.Callable[[], t.Coroutine[t.Any, t.Any, t.Optional['DidDocument']]]


class _DidBaseCache:
    def __init__(self, stale_ttl: t.Optional[int] = None, max_ttl: t.Optional[int] = None) -> None:
        self.stale_ttl = stale_ttl or _DEFAULT_STALE_TTL
        self.max_ttl = max_ttl or _DEFAULT_MAX_TTL


class DidBaseCache(_DidBaseCache, ABC):
    """Abstract DID Cache.

    Args:
        stale_ttl: Stale TTL in seconds. Default is 1 hour.
        max_ttl: Max TTL in seconds. Default is 1 day.
    """

    def __init__(self, stale_ttl: t.Optional[int] = None, max_ttl: t.Optional[int] = None) -> None:
        super().__init__(stale_ttl, max_ttl)

    @abstractmethod
    def get(self, did: str) -> t.Optional['CachedDidResult']:
        """Get cached DID.

        Args:
            did: DID.

        Returns:
            :obj:`CachedDidResult`: Cached DID result or ``None`` if not found.
        """
        raise NotImplementedError

    @abstractmethod
    def set(self, did: str, document: 'DidDocument') -> None:
        """Set cached DID.

        Args:
            did: DID.
            document: DID document.
        """
        raise NotImplementedError

    @abstractmethod
    def refresh(self, did: str, get_doc_callback: GetDocCallback) -> None:
        """Refresh cached DID.

        Args:
            did: DID.
            get_doc_callback: Get DID document callback.
        """
        raise NotImplementedError

    @abstractmethod
    def delete(self, did: str) -> None:
        """Delete cached DID.

        Args:
            did: DID.
        """
        raise NotImplementedError

    @abstractmethod
    def clear(self) -> None:
        """Clear cached DIDs.

        Note:
            This method is used to clear all cached DIDs.
        """
        raise NotImplementedError


class AsyncDidBaseCache(_DidBaseCache, ABC):
    """Asynchronous Abstract DID Cache.

    Args:
        stale_ttl: Stale TTL in seconds. Default is 1 hour.
        max_ttl: Max TTL in seconds. Default is 1 day.
    """

    def __init__(self, stale_ttl: t.Optional[int] = None, max_ttl: t.Optional[int] = None) -> None:
        super().__init__(stale_ttl, max_ttl)

    @abstractmethod
    async def get(self, did: str) -> t.Optional['CachedDidResult']:
        """Get cached DID.

        Args:
            did: DID.

        Returns:
            :obj:`CachedDidResult`: Cached DID result or ``None`` if not found.
        """
        raise NotImplementedError

    @abstractmethod
    async def set(self, did: str, document: 'DidDocument') -> None:
        """Set cached DID.

        Args:
            did: DID.
            document: DID document.
        """
        raise NotImplementedError

    @abstractmethod
    async def refresh(self, did: str, get_doc_callback: AsyncGetDocCallback) -> None:
        """Refresh cached DID.

        Args:
            did: DID.
            get_doc_callback: Get DID document callback.
        """
        raise NotImplementedError

    @abstractmethod
    async def delete(self, did: str) -> None:
        """Delete cached DID.

        Args:
            did: DID.
        """
        raise NotImplementedError

    @abstractmethod
    async def clear(self) -> None:
        """Clear cached DIDs.

        Note:
            This method is used to clear all cached DIDs.
        """
        raise NotImplementedError



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.server.describeServer`."""

    available_user_domains: t.List[str]  #: List of domain suffixes that can be used in account handles.
    did: str  #: Did.
    invite_code_required: t.Optional[
        bool
    ] = None  #: If true, an invite code must be supplied to create an account on this instance.
    links: t.Optional['models.ComAtprotoServerDescribeServer.Links'] = None  #: URLs of service policy documents.
    phone_verification_required: t.Optional[
        bool
    ] = None  #: If true, a phone verification token must be supplied to create an account on this instance.


class Links(base.ModelBase):
    """Definition model for :obj:`com.atproto.server.describeServer`."""

    privacy_policy: t.Optional[str] = None  #: Privacy policy.
    terms_of_service: t.Optional[str] = None  #: Terms of service.

    py_type: te.Literal['com.atproto.server.describeServer#links'] = Field(
        default='com.atproto.server.describeServer#links', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.server.updateEmail`."""

    email: str  #: Email.
    token: t.Optional[
        str
    ] = None  #: Requires a token from com.atproto.sever.requestEmailUpdate if the account's email has been confirmed.


class DataDict(te.TypedDict):
    email: str  #: Email.
    token: te.NotRequired[
        t.Optional[str]
    ]  #: Requires a token from com.atproto.sever.requestEmailUpdate if the account's email has been confirmed.



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing_extensions as te

from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.admin.updateAccountPassword`."""

    did: str  #: Did.
    password: str  #: Password.


class DataDict(te.TypedDict):
    did: str  #: Did.
    password: str  #: Password.



import json
import types
import typing as t

import typing_extensions as te
from pydantic import ValidationError

from atproto_client import models
from atproto_client.exceptions import (
    ModelError,
    ModelFieldNotFoundError,
)
from atproto_client.models.base import ModelBase
from atproto_client.models.blob_ref import BlobRef
from atproto_client.models.dot_dict import DotDict
from atproto_client.models.type_conversion import RECORD_TYPE_TO_MODEL_CLASS
from atproto_client.models.unknown_type import UnknownRecordType

if t.TYPE_CHECKING:
    from atproto_client.request import Response

M = t.TypeVar('M')
ModelData: te.TypeAlias = t.Union[
    M, t.Dict[str, t.Any], None
]  # we assume that dict is JSON object. not list or primitive

_TYPE_SERVICE_FIELD = '$type'


def get_or_create(
    model_data: ModelData[M], model: t.Optional[t.Type[M]] = None, *, strict: bool = True
) -> t.Optional[t.Union[M, UnknownRecordType, DotDict]]:
    """Get model instance from raw data.

    Note:
        The record could have additional fields and be completely custom.
        For example, third-party bsky clients add a "via"
        field to indicate that it was posted using a not official client.
        Such records are corresponding to the lexicon, but have additional fields.
        This is called "extended record".
        Extended records will be decoded to proper models with extra, non-typehinted fields available only in runtime.
        Unknown record types will be decoded to :obj:`atproto.xrpc_client.models.base.DotDict`.

    Note:
        By default, the method raises an exception on custom models if you have passed the expected model.
        To fall back to a :obj:`atproto.xrpc_client.models.base.DotDict` type, disable strict mode using the argument.

    Note:
        Model auto-resolve works only with a Record type for now.

    Args:
        model_data: Raw data.
        model: Class of the model or any another type. If None, it will be resolved automatically.
        strict: Disable fallback to dictionary (:obj:`atproto.xrpc_client.models.base.DotDict`)
            if it can't be properly deserialized in provided `model`. Will raise the exception instead.

    Returns:
        Instance of :obj:`model` or :obj:`None` or
        :obj:`atproto.xrpc_client.models.dot_dict.DotDict` if `strict` is disabled.
    """
    try:
        model_instance = _get_or_create(model_data, model, strict=strict)
        if strict and model_instance is not None and (not model or not isinstance(model_instance, model)):
            raise ModelError(f"Can't properly parse model of type {model}")

        return model_instance
    except Exception as e:  # noqa: BLE001
        if strict or not isinstance(model_data, dict):
            raise e

        return DotDict(model_data)


def _get_or_create(
    model_data: ModelData[M], model: t.Optional[t.Type[M]], *, strict: bool
) -> t.Optional[t.Union[M, UnknownRecordType, DotDict]]:
    if model_data is None:
        return None

    if model is None:
        if not isinstance(model_data, dict):
            return None

        # we are sure that this is dict because of check above
        model_data = t.cast(t.Dict[str, t.Any], model_data)

        # resolve a record model by type and try to deserialize
        record_type: t.Any = model_data.get(_TYPE_SERVICE_FIELD)
        if record_type not in RECORD_TYPE_TO_MODEL_CLASS:
            return None

        # now we are sure that this is str because it's in RECORD_TYPE_TO_MODEL_CLASS
        record_type = t.cast(str, record_type)

        return get_or_create(model_data, RECORD_TYPE_TO_MODEL_CLASS[record_type], strict=strict)

    if isinstance(model_data, model):
        return model_data

    try:
        return model(**model_data)
    except ValidationError as e:
        raise ModelError(str(e)) from e


def get_response_model(response: 'Response', model: t.Type[M]) -> M:
    if model is bool:
        # Could not be False? Because the exception with errors will be raised from the server
        return t.cast(M, response.success)
    if model is bytes:
        return t.cast(M, response.content)

    if not isinstance(response.content, dict):
        raise ModelError("Can't properly parse response model because JSON is expected in response")

    # casting to M because of enabled strict mode
    return t.cast(M, get_or_create(response.content, model, strict=True))


def get_model_as_dict(model: t.Union[DotDict, BlobRef, ModelBase]) -> t.Dict[str, t.Any]:
    if isinstance(model, DotDict):
        return model.to_dict()

    return model.model_dump(exclude_none=True, by_alias=True)


def get_model_as_json(model: t.Union[DotDict, BlobRef, ModelBase]) -> str:
    if isinstance(model, DotDict):
        return json.dumps(get_model_as_dict(model))

    return model.model_dump_json(exclude_none=True, by_alias=True)


def is_json(json_data: t.Union[str, bytes]) -> bool:
    if isinstance(json_data, bytes):
        json_data.decode('UTF-8')

    try:
        json.loads(json_data)
        return True
    except:  # noqa
        return False


def is_record_type(model: t.Union[ModelBase, DotDict], expected_type: t.Union[str, types.ModuleType]) -> bool:
    """Verify that the model is the expected Record type.

    Args:
        model: Model to be verified.
        expected_type: Excepted type.
            Could be NSID or Python Module.

    Example:
        >>> from atproto import Client, models
        >>> from atproto_client.models import ids, is_record_type
        >>> client = Client()
        >>> client.login('username', 'pass')
        >>> record = client.com.atproto.repo.get_record(...)
        >>> # using NSID:
        >>> is_record_type(record.value, ids.AppBskyFeedPost)
        >>> # using Python module:
        >>> is_record_type(record.value, models.AppBskyFeedPost)

    Returns:
        :obj:`bool`: Is record or not.
    """
    if isinstance(expected_type, types.ModuleType):
        # for now, all records are defined in the Record class
        # TODO(MarshalX): remove backward compatibility for Main
        if not hasattr(expected_type, 'Main') or not hasattr(expected_type, 'Record'):
            return False

        if hasattr(expected_type, 'Main'):
            expected_type = expected_type.Main.model_fields['py_type'].default

        if hasattr(expected_type, 'Record'):
            expected_type = expected_type.Record.model_fields['py_type'].default

    if isinstance(model, DotDict):  # custom record
        try:
            return expected_type == model[_TYPE_SERVICE_FIELD]
        except ModelFieldNotFoundError:
            return False

    return expected_type == model.py_type


def create_strong_ref(model: ModelBase) -> models.ComAtprotoRepoStrongRef.Main:
    """Create a strong ref from the model.

    Args:
        model: Any model with `cid` and `uri` fields.

    Example:
        >>> from atproto import Client
        >>> client = Client()
        >>> client.login('my-handle', 'my-password')
        >>> response = client.send_post(text='Hello World from Python!')
        >>> strong_ref = create_strong_ref(response)

    Returns:
        :obj:`atproto.xrpc_client.models.com.atproto.repo.strong_ref.Main`: Strong ref.
    """
    if hasattr(model, 'cid') and hasattr(model, 'uri'):
        return models.ComAtprotoRepoStrongRef.Main(cid=model.cid, uri=model.uri)

    raise ModelError('Could not create strong ref from model')



from cryptography.hazmat.primitives.asymmetric.ec import SECP256K1

from atproto_crypto.algs.base_alg import AlgBase
from atproto_crypto.consts import SECP256K1_CURVE_ORDER, SECP256K1_JWT_ALG


class Secp256k1(AlgBase):
    """K256 algorithm AKA secp256k1 AKA ES256K."""

    NAME = SECP256K1_JWT_ALG

    def __init__(self) -> None:
        super().__init__(SECP256K1(), SECP256K1_CURVE_ORDER)



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
from atproto_client.models import base


class Response(base.ResponseModelBase):
    """Output data model for :obj:`app.bsky.feed.describeFeedGenerator`."""

    did: str  #: Did.
    feeds: t.List['models.AppBskyFeedDescribeFeedGenerator.Feed']  #: Feeds.
    links: t.Optional['models.AppBskyFeedDescribeFeedGenerator.Links'] = None  #: Links.


class Feed(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.describeFeedGenerator`."""

    uri: str  #: Uri.

    py_type: te.Literal['app.bsky.feed.describeFeedGenerator#feed'] = Field(
        default='app.bsky.feed.describeFeedGenerator#feed', alias='$type', frozen=True
    )


class Links(base.ModelBase):
    """Definition model for :obj:`app.bsky.feed.describeFeedGenerator`."""

    privacy_policy: t.Optional[str] = None  #: Privacy policy.
    terms_of_service: t.Optional[str] = None  #: Terms of service.

    py_type: te.Literal['app.bsky.feed.describeFeedGenerator#links'] = Field(
        default='app.bsky.feed.describeFeedGenerator#links', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te

from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.repo.deleteRecord`."""

    collection: str  #: The NSID of the record collection.
    repo: str  #: The handle or DID of the repo (aka, current account).
    rkey: str  #: The Record Key.
    swap_commit: t.Optional[str] = None  #: Compare and swap with the previous commit by CID.
    swap_record: t.Optional[str] = None  #: Compare and swap with the previous record by CID.


class DataDict(te.TypedDict):
    collection: str  #: The NSID of the record collection.
    repo: str  #: The handle or DID of the repo (aka, current account).
    rkey: str  #: The Record Key.
    swap_commit: te.NotRequired[t.Optional[str]]  #: Compare and swap with the previous commit by CID.
    swap_record: te.NotRequired[t.Optional[str]]  #: Compare and swap with the previous record by CID.



import typing as t
from dataclasses import dataclass
from pathlib import Path

from atproto_core.nsid import NSID
from atproto_lexicon.models import (
    LexDefinition,
    LexDefinitionType,
    LexiconDoc,
    LexRecord,
    LexXrpcProcedure,
    LexXrpcQuery,
)
from atproto_lexicon.parser import lexicon_parse_dir

_VALID_LEX_DEF_TYPES = {LexDefinitionType.QUERY, LexDefinitionType.PROCEDURE, LexDefinitionType.RECORD}


def _filter_namespace_valid_definitions(definitions: t.Dict[str, LexDefinition]) -> t.Dict[str, LexDefinition]:
    result = {}

    for def_name, def_content in definitions.items():
        if def_content.type in _VALID_LEX_DEF_TYPES:
            result[def_name] = def_content

    return result


def get_definition_by_name(name: str, defs: t.Dict[str, LexDefinition]) -> LexDefinition:
    if name in defs:
        return defs[name]

    return defs['main']


@dataclass
class ObjectInfo:
    name: str
    nsid: NSID


@dataclass
class ProcedureInfo(ObjectInfo):
    definition: LexXrpcProcedure


@dataclass
class QueryInfo(ObjectInfo):
    definition: LexXrpcQuery


MethodInfo = t.Union[ProcedureInfo, QueryInfo]


@dataclass
class RecordInfo(ObjectInfo):
    definition: LexRecord


def _enrich_namespace_tree(root: dict, nsid: NSID, defs: t.Dict[str, LexDefinition]) -> None:
    root_node: t.Union[dict, list] = root

    segments_count = len(nsid.segments)
    for path_level, segment in enumerate(nsid.segments):
        # if method
        if path_level == segments_count - 1:
            definition = get_definition_by_name(segment, defs)

            model_class: t.Type[ObjectInfo]
            if definition.type is LexDefinitionType.PROCEDURE:
                model_class = ProcedureInfo
            elif definition.type is LexDefinitionType.QUERY:
                model_class = QueryInfo
            elif definition.type is LexDefinitionType.RECORD:
                model_class = RecordInfo
            else:
                raise RuntimeError(f'Unknown definition type: {definition.type}')

            # TODO(MarshalX): fake records as namespaces with methods to be able to reuse code of generator?

            if model_class:
                root_node.append(model_class(name=segment, nsid=nsid, definition=definition))

            continue

        if segment not in root_node:
            # if end of method's path
            if path_level == segments_count - 2:
                root_node[segment] = []
            else:
                root_node[segment] = {}
        root_node = root_node[segment]


def build_namespace_tree(lexicons: t.List[LexiconDoc]) -> dict:
    namespace_tree = {}
    for lexicon in lexicons:
        defs = _filter_namespace_valid_definitions(lexicon.defs)
        if not defs:
            continue

        nsid = NSID.from_str(lexicon.id)
        _enrich_namespace_tree(namespace_tree, nsid, defs)

    return namespace_tree


def build_namespaces(lexicon_dir: t.Optional[Path] = None) -> dict:
    lexicons = lexicon_parse_dir(lexicon_dir)
    return build_namespace_tree(lexicons)


if __name__ == '__main__':
    build_namespaces()



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client import models
    from atproto_client.models.unknown_type import UnknownType
from atproto_client.models import base


class StatusAttr(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    applied: bool  #: Applied.
    ref: t.Optional[str] = None  #: Ref.

    py_type: te.Literal['com.atproto.admin.defs#statusAttr'] = Field(
        default='com.atproto.admin.defs#statusAttr', alias='$type', frozen=True
    )


class ModEventView(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    created_at: str  #: Created at.
    created_by: str  #: Created by.
    event: te.Annotated[
        t.Union[
            'models.ComAtprotoAdminDefs.ModEventTakedown',
            'models.ComAtprotoAdminDefs.ModEventReverseTakedown',
            'models.ComAtprotoAdminDefs.ModEventComment',
            'models.ComAtprotoAdminDefs.ModEventReport',
            'models.ComAtprotoAdminDefs.ModEventLabel',
            'models.ComAtprotoAdminDefs.ModEventAcknowledge',
            'models.ComAtprotoAdminDefs.ModEventEscalate',
            'models.ComAtprotoAdminDefs.ModEventMute',
            'models.ComAtprotoAdminDefs.ModEventEmail',
            'models.ComAtprotoAdminDefs.ModEventResolveAppeal',
        ],
        Field(discriminator='py_type'),
    ]  #: Event.
    id: int  #: Id.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    subject_blob_cids: t.List[str]  #: Subject blob cids.
    creator_handle: t.Optional[str] = None  #: Creator handle.
    subject_handle: t.Optional[str] = None  #: Subject handle.

    py_type: te.Literal['com.atproto.admin.defs#modEventView'] = Field(
        default='com.atproto.admin.defs#modEventView', alias='$type', frozen=True
    )


class ModEventViewDetail(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    created_at: str  #: Created at.
    created_by: str  #: Created by.
    event: te.Annotated[
        t.Union[
            'models.ComAtprotoAdminDefs.ModEventTakedown',
            'models.ComAtprotoAdminDefs.ModEventReverseTakedown',
            'models.ComAtprotoAdminDefs.ModEventComment',
            'models.ComAtprotoAdminDefs.ModEventReport',
            'models.ComAtprotoAdminDefs.ModEventLabel',
            'models.ComAtprotoAdminDefs.ModEventAcknowledge',
            'models.ComAtprotoAdminDefs.ModEventEscalate',
            'models.ComAtprotoAdminDefs.ModEventMute',
            'models.ComAtprotoAdminDefs.ModEventEmail',
            'models.ComAtprotoAdminDefs.ModEventResolveAppeal',
        ],
        Field(discriminator='py_type'),
    ]  #: Event.
    id: int  #: Id.
    subject: te.Annotated[
        t.Union[
            'models.ComAtprotoAdminDefs.RepoView',
            'models.ComAtprotoAdminDefs.RepoViewNotFound',
            'models.ComAtprotoAdminDefs.RecordView',
            'models.ComAtprotoAdminDefs.RecordViewNotFound',
        ],
        Field(discriminator='py_type'),
    ]  #: Subject.
    subject_blobs: t.List['models.ComAtprotoAdminDefs.BlobView']  #: Subject blobs.

    py_type: te.Literal['com.atproto.admin.defs#modEventViewDetail'] = Field(
        default='com.atproto.admin.defs#modEventViewDetail', alias='$type', frozen=True
    )


class ReportView(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    created_at: str  #: Created at.
    id: int  #: Id.
    reason_type: 'models.ComAtprotoModerationDefs.ReasonType'  #: Reason type.
    reported_by: str  #: Reported by.
    resolved_by_action_ids: t.List[int]  #: Resolved by action ids.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    comment: t.Optional[str] = None  #: Comment.
    subject_repo_handle: t.Optional[str] = None  #: Subject repo handle.

    py_type: te.Literal['com.atproto.admin.defs#reportView'] = Field(
        default='com.atproto.admin.defs#reportView', alias='$type', frozen=True
    )


class SubjectStatusView(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    created_at: str  #: Timestamp referencing the first moderation status impacting event was emitted on the subject.
    id: int  #: Id.
    review_state: 'models.ComAtprotoAdminDefs.SubjectReviewState'  #: Review state.
    subject: te.Annotated[
        t.Union['models.ComAtprotoAdminDefs.RepoRef', 'models.ComAtprotoRepoStrongRef.Main'],
        Field(discriminator='py_type'),
    ]  #: Subject.
    updated_at: str  #: Timestamp referencing when the last update was made to the moderation status of the subject.
    appealed: t.Optional[
        bool
    ] = None  #: True indicates that the a previously taken moderator action was appealed against, by the author of the content. False indicates last appeal was resolved by moderators.
    comment: t.Optional[str] = None  #: Sticky comment on the subject.
    last_appealed_at: t.Optional[
        str
    ] = None  #: Timestamp referencing when the author of the subject appealed a moderation action.
    last_reported_at: t.Optional[str] = None  #: Last reported at.
    last_reviewed_at: t.Optional[str] = None  #: Last reviewed at.
    last_reviewed_by: t.Optional[str] = None  #: Last reviewed by.
    mute_until: t.Optional[str] = None  #: Mute until.
    subject_blob_cids: t.Optional[t.List[str]] = None  #: Subject blob cids.
    subject_repo_handle: t.Optional[str] = None  #: Subject repo handle.
    suspend_until: t.Optional[str] = None  #: Suspend until.
    tags: t.Optional[t.List[str]] = None  #: Tags.
    takendown: t.Optional[bool] = None  #: Takendown.

    py_type: te.Literal['com.atproto.admin.defs#subjectStatusView'] = Field(
        default='com.atproto.admin.defs#subjectStatusView', alias='$type', frozen=True
    )


class ReportViewDetail(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    created_at: str  #: Created at.
    id: int  #: Id.
    reason_type: 'models.ComAtprotoModerationDefs.ReasonType'  #: Reason type.
    reported_by: str  #: Reported by.
    resolved_by_actions: t.List['models.ComAtprotoAdminDefs.ModEventView']  #: Resolved by actions.
    subject: te.Annotated[
        t.Union[
            'models.ComAtprotoAdminDefs.RepoView',
            'models.ComAtprotoAdminDefs.RepoViewNotFound',
            'models.ComAtprotoAdminDefs.RecordView',
            'models.ComAtprotoAdminDefs.RecordViewNotFound',
        ],
        Field(discriminator='py_type'),
    ]  #: Subject.
    comment: t.Optional[str] = None  #: Comment.
    subject_status: t.Optional['models.ComAtprotoAdminDefs.SubjectStatusView'] = None  #: Subject status.

    py_type: te.Literal['com.atproto.admin.defs#reportViewDetail'] = Field(
        default='com.atproto.admin.defs#reportViewDetail', alias='$type', frozen=True
    )


class RepoView(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    did: str  #: Did.
    handle: str  #: Handle.
    indexed_at: str  #: Indexed at.
    moderation: 'models.ComAtprotoAdminDefs.Moderation'  #: Moderation.
    related_records: t.List['UnknownType']  #: Related records.
    email: t.Optional[str] = None  #: Email.
    invite_note: t.Optional[str] = None  #: Invite note.
    invited_by: t.Optional['models.ComAtprotoServerDefs.InviteCode'] = None  #: Invited by.
    invites_disabled: t.Optional[bool] = None  #: Invites disabled.

    py_type: te.Literal['com.atproto.admin.defs#repoView'] = Field(
        default='com.atproto.admin.defs#repoView', alias='$type', frozen=True
    )


class RepoViewDetail(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    did: str  #: Did.
    handle: str  #: Handle.
    indexed_at: str  #: Indexed at.
    moderation: 'models.ComAtprotoAdminDefs.ModerationDetail'  #: Moderation.
    related_records: t.List['UnknownType']  #: Related records.
    email: t.Optional[str] = None  #: Email.
    email_confirmed_at: t.Optional[str] = None  #: Email confirmed at.
    invite_note: t.Optional[str] = None  #: Invite note.
    invited_by: t.Optional['models.ComAtprotoServerDefs.InviteCode'] = None  #: Invited by.
    invites: t.Optional[t.List['models.ComAtprotoServerDefs.InviteCode']] = None  #: Invites.
    invites_disabled: t.Optional[bool] = None  #: Invites disabled.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.

    py_type: te.Literal['com.atproto.admin.defs#repoViewDetail'] = Field(
        default='com.atproto.admin.defs#repoViewDetail', alias='$type', frozen=True
    )


class AccountView(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    did: str  #: Did.
    handle: str  #: Handle.
    indexed_at: str  #: Indexed at.
    email: t.Optional[str] = None  #: Email.
    email_confirmed_at: t.Optional[str] = None  #: Email confirmed at.
    invite_note: t.Optional[str] = None  #: Invite note.
    invited_by: t.Optional['models.ComAtprotoServerDefs.InviteCode'] = None  #: Invited by.
    invites: t.Optional[t.List['models.ComAtprotoServerDefs.InviteCode']] = None  #: Invites.
    invites_disabled: t.Optional[bool] = None  #: Invites disabled.
    related_records: t.Optional[t.List['UnknownType']] = None  #: Related records.

    py_type: te.Literal['com.atproto.admin.defs#accountView'] = Field(
        default='com.atproto.admin.defs#accountView', alias='$type', frozen=True
    )


class RepoViewNotFound(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    did: str  #: Did.

    py_type: te.Literal['com.atproto.admin.defs#repoViewNotFound'] = Field(
        default='com.atproto.admin.defs#repoViewNotFound', alias='$type', frozen=True
    )


class RepoRef(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    did: str  #: Did.

    py_type: te.Literal['com.atproto.admin.defs#repoRef'] = Field(
        default='com.atproto.admin.defs#repoRef', alias='$type', frozen=True
    )


class RepoBlobRef(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    cid: str  #: Cid.
    did: str  #: Did.
    record_uri: t.Optional[str] = None  #: Record uri.

    py_type: te.Literal['com.atproto.admin.defs#repoBlobRef'] = Field(
        default='com.atproto.admin.defs#repoBlobRef', alias='$type', frozen=True
    )


class RecordView(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    blob_cids: t.List[str]  #: Blob cids.
    cid: str  #: Cid.
    indexed_at: str  #: Indexed at.
    moderation: 'models.ComAtprotoAdminDefs.Moderation'  #: Moderation.
    repo: 'models.ComAtprotoAdminDefs.RepoView'  #: Repo.
    uri: str  #: Uri.
    value: 'UnknownType'  #: Value.

    py_type: te.Literal['com.atproto.admin.defs#recordView'] = Field(
        default='com.atproto.admin.defs#recordView', alias='$type', frozen=True
    )


class RecordViewDetail(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    blobs: t.List['models.ComAtprotoAdminDefs.BlobView']  #: Blobs.
    cid: str  #: Cid.
    indexed_at: str  #: Indexed at.
    moderation: 'models.ComAtprotoAdminDefs.ModerationDetail'  #: Moderation.
    repo: 'models.ComAtprotoAdminDefs.RepoView'  #: Repo.
    uri: str  #: Uri.
    value: 'UnknownType'  #: Value.
    labels: t.Optional[t.List['models.ComAtprotoLabelDefs.Label']] = None  #: Labels.

    py_type: te.Literal['com.atproto.admin.defs#recordViewDetail'] = Field(
        default='com.atproto.admin.defs#recordViewDetail', alias='$type', frozen=True
    )


class RecordViewNotFound(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    uri: str  #: Uri.

    py_type: te.Literal['com.atproto.admin.defs#recordViewNotFound'] = Field(
        default='com.atproto.admin.defs#recordViewNotFound', alias='$type', frozen=True
    )


class Moderation(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    subject_status: t.Optional['models.ComAtprotoAdminDefs.SubjectStatusView'] = None  #: Subject status.

    py_type: te.Literal['com.atproto.admin.defs#moderation'] = Field(
        default='com.atproto.admin.defs#moderation', alias='$type', frozen=True
    )


class ModerationDetail(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    subject_status: t.Optional['models.ComAtprotoAdminDefs.SubjectStatusView'] = None  #: Subject status.

    py_type: te.Literal['com.atproto.admin.defs#moderationDetail'] = Field(
        default='com.atproto.admin.defs#moderationDetail', alias='$type', frozen=True
    )


class BlobView(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    cid: str  #: Cid.
    created_at: str  #: Created at.
    mime_type: str  #: Mime type.
    size: int  #: Size.
    details: t.Optional[
        te.Annotated[
            t.Union['models.ComAtprotoAdminDefs.ImageDetails', 'models.ComAtprotoAdminDefs.VideoDetails'],
            Field(default=None, discriminator='py_type'),
        ]
    ] = None  #: Details.
    moderation: t.Optional['models.ComAtprotoAdminDefs.Moderation'] = None  #: Moderation.

    py_type: te.Literal['com.atproto.admin.defs#blobView'] = Field(
        default='com.atproto.admin.defs#blobView', alias='$type', frozen=True
    )


class ImageDetails(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    height: int  #: Height.
    width: int  #: Width.

    py_type: te.Literal['com.atproto.admin.defs#imageDetails'] = Field(
        default='com.atproto.admin.defs#imageDetails', alias='$type', frozen=True
    )


class VideoDetails(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    height: int  #: Height.
    length: int  #: Length.
    width: int  #: Width.

    py_type: te.Literal['com.atproto.admin.defs#videoDetails'] = Field(
        default='com.atproto.admin.defs#videoDetails', alias='$type', frozen=True
    )


SubjectReviewState = t.Union[
    'models.ComAtprotoAdminDefs.ReviewOpen',
    'models.ComAtprotoAdminDefs.ReviewEscalated',
    'models.ComAtprotoAdminDefs.ReviewClosed',
]  #: Subject review state

ReviewOpen = te.Literal[
    'com.atproto.admin.defs#reviewOpen'
]  #: Moderator review status of a subject: Open. Indicates that the subject needs to be reviewed by a moderator

ReviewEscalated = te.Literal[
    'com.atproto.admin.defs#reviewEscalated'
]  #: Moderator review status of a subject: Escalated. Indicates that the subject was escalated for review by a moderator

ReviewClosed = te.Literal[
    'com.atproto.admin.defs#reviewClosed'
]  #: Moderator review status of a subject: Closed. Indicates that the subject was already reviewed and resolved by a moderator


class ModEventTakedown(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Take down a subject permanently or temporarily."""

    comment: t.Optional[str] = None  #: Comment.
    duration_in_hours: t.Optional[
        int
    ] = None  #: Indicates how long the takedown should be in effect before automatically expiring.

    py_type: te.Literal['com.atproto.admin.defs#modEventTakedown'] = Field(
        default='com.atproto.admin.defs#modEventTakedown', alias='$type', frozen=True
    )


class ModEventReverseTakedown(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Revert take down action on a subject."""

    comment: t.Optional[str] = None  #: Describe reasoning behind the reversal.

    py_type: te.Literal['com.atproto.admin.defs#modEventReverseTakedown'] = Field(
        default='com.atproto.admin.defs#modEventReverseTakedown', alias='$type', frozen=True
    )


class ModEventResolveAppeal(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Resolve appeal on a subject."""

    comment: t.Optional[str] = None  #: Describe resolution.

    py_type: te.Literal['com.atproto.admin.defs#modEventResolveAppeal'] = Field(
        default='com.atproto.admin.defs#modEventResolveAppeal', alias='$type', frozen=True
    )


class ModEventComment(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Add a comment to a subject."""

    comment: str  #: Comment.
    sticky: t.Optional[bool] = None  #: Make the comment persistent on the subject.

    py_type: te.Literal['com.atproto.admin.defs#modEventComment'] = Field(
        default='com.atproto.admin.defs#modEventComment', alias='$type', frozen=True
    )


class ModEventReport(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Report a subject."""

    report_type: 'models.ComAtprotoModerationDefs.ReasonType'  #: Report type.
    comment: t.Optional[str] = None  #: Comment.

    py_type: te.Literal['com.atproto.admin.defs#modEventReport'] = Field(
        default='com.atproto.admin.defs#modEventReport', alias='$type', frozen=True
    )


class ModEventLabel(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Apply/Negate labels on a subject."""

    create_label_vals: t.List[str]  #: Create label vals.
    negate_label_vals: t.List[str]  #: Negate label vals.
    comment: t.Optional[str] = None  #: Comment.

    py_type: te.Literal['com.atproto.admin.defs#modEventLabel'] = Field(
        default='com.atproto.admin.defs#modEventLabel', alias='$type', frozen=True
    )


class ModEventAcknowledge(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    comment: t.Optional[str] = None  #: Comment.

    py_type: te.Literal['com.atproto.admin.defs#modEventAcknowledge'] = Field(
        default='com.atproto.admin.defs#modEventAcknowledge', alias='$type', frozen=True
    )


class ModEventEscalate(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    comment: t.Optional[str] = None  #: Comment.

    py_type: te.Literal['com.atproto.admin.defs#modEventEscalate'] = Field(
        default='com.atproto.admin.defs#modEventEscalate', alias='$type', frozen=True
    )


class ModEventMute(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Mute incoming reports on a subject."""

    duration_in_hours: int  #: Indicates how long the subject should remain muted.
    comment: t.Optional[str] = None  #: Comment.

    py_type: te.Literal['com.atproto.admin.defs#modEventMute'] = Field(
        default='com.atproto.admin.defs#modEventMute', alias='$type', frozen=True
    )


class ModEventUnmute(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Unmute action on a subject."""

    comment: t.Optional[str] = None  #: Describe reasoning behind the reversal.

    py_type: te.Literal['com.atproto.admin.defs#modEventUnmute'] = Field(
        default='com.atproto.admin.defs#modEventUnmute', alias='$type', frozen=True
    )


class ModEventEmail(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Keep a log of outgoing email to a user."""

    subject_line: str  #: The subject line of the email sent to the user.
    comment: t.Optional[str] = None  #: Additional comment about the outgoing comm.

    py_type: te.Literal['com.atproto.admin.defs#modEventEmail'] = Field(
        default='com.atproto.admin.defs#modEventEmail', alias='$type', frozen=True
    )


class ModEventTag(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`. Add/Remove a tag on a subject."""

    add: t.List[str]  #: Tags to be added to the subject. If already exists, won't be duplicated.
    remove: t.List[str]  #: Tags to be removed to the subject. Ignores a tag If it doesn't exist, won't be duplicated.
    comment: t.Optional[str] = None  #: Additional comment about added/removed tags.

    py_type: te.Literal['com.atproto.admin.defs#modEventTag'] = Field(
        default='com.atproto.admin.defs#modEventTag', alias='$type', frozen=True
    )


class CommunicationTemplateView(base.ModelBase):
    """Definition model for :obj:`com.atproto.admin.defs`."""

    content_markdown: str  #: Subject of the message, used in emails.
    created_at: str  #: Created at.
    disabled: bool  #: Disabled.
    id: str  #: Id.
    last_updated_by: str  #: DID of the user who last updated the template.
    name: str  #: Name of the template.
    updated_at: str  #: Updated at.
    subject: t.Optional[str] = None  #: Content of the template, can contain markdown and variable placeholders.

    py_type: te.Literal['com.atproto.admin.defs#communicationTemplateView'] = Field(
        default='com.atproto.admin.defs#communicationTemplateView', alias='$type', frozen=True
    )



##################################################################
# THIS IS THE AUTO-GENERATED CODE. DON'T EDIT IT BY HANDS!
# Copyright (C) 2023 Ilya (Marshal) <https://github.com/MarshalX>.
# This file is part of Python atproto SDK. Licenced under MIT.
##################################################################


import typing as t

import typing_extensions as te
from pydantic import Field

if t.TYPE_CHECKING:
    from atproto_client.models.unknown_type import UnknownInputType
from atproto_client.models import base


class Data(base.DataModelBase):
    """Input data model for :obj:`com.atproto.repo.createRecord`."""

    collection: str  #: The NSID of the record collection.
    record: 'UnknownInputType'  #: The record itself. Must contain a $type field.
    repo: str  #: The handle or DID of the repo (aka, current account).
    rkey: t.Optional[str] = Field(default=None, max_length=15)  #: The Record Key.
    swap_commit: t.Optional[str] = None  #: Compare and swap with the previous commit by CID.
    validate_: t.Optional[bool] = None  #: Can be set to 'false' to skip Lexicon schema validation of record data.


class DataDict(te.TypedDict):
    collection: str  #: The NSID of the record collection.
    record: 'UnknownInputType'  #: The record itself. Must contain a $type field.
    repo: str  #: The handle or DID of the repo (aka, current account).
    rkey: te.NotRequired[t.Optional[str]]  #: The Record Key.
    swap_commit: te.NotRequired[t.Optional[str]]  #: Compare and swap with the previous commit by CID.
    validate: te.NotRequired[
        t.Optional[bool]
    ]  #: Can be set to 'false' to skip Lexicon schema validation of record data.


class Response(base.ResponseModelBase):
    """Output data model for :obj:`com.atproto.repo.createRecord`."""

    cid: str  #: Cid.
    uri: str  #: Uri.



import typing as t
from dataclasses import dataclass
from enum import Enum

import typing_extensions as te

if t.TYPE_CHECKING:
    from atproto_client import models


class SessionEvent(Enum):
    IMPORT = 'import'
    CREATE = 'creat'
    REFRESH = 'refresh'


_SESSION_STRING_SEPARATOR: te.Final[te.Literal[':::']] = ':::'


@dataclass
class Session:
    handle: str
    did: str
    access_jwt: str
    refresh_jwt: str

    def __repr__(self) -> str:
        return f'<Session handle={self.handle} did={self.did}>'

    def __str__(self) -> str:
        return self.encode()

    def encode(self) -> str:
        payload = [
            self.handle,
            self.did,
            self.access_jwt,
            self.refresh_jwt,
        ]
        return _SESSION_STRING_SEPARATOR.join(payload)

    @classmethod
    def decode(cls, session_string: str) -> 'Session':
        handle, did, access_jwt, refresh_jwt = session_string.split(_SESSION_STRING_SEPARATOR)
        return cls(handle, did, access_jwt, refresh_jwt)

    def copy(self) -> 'Session':
        return Session(self.handle, self.did, self.access_jwt, self.refresh_jwt)

    #: Alias for :attr:`encode`
    export = encode


SessionResponse: te.TypeAlias = t.Union[
    'models.ComAtprotoServerCreateSession.Response',
    'models.ComAtprotoServerRefreshSession.Response',
    'Session',
]

SessionChangeCallback = t.Callable[[SessionEvent, Session], None]
AsyncSessionChangeCallback = t.Callable[[SessionEvent, Session], t.Coroutine[t.Any, t.Any, None]]


class SessionString(Session):
    def __init_subclass__(cls, *args, **kwargs: t.Any) -> None:
        import warnings

        warnings.warn('SessionString class is deprecated. Use Session class instead.', DeprecationWarning, stacklevel=2)
        super().__init_subclass__(*args, **kwargs)

    def __init__(self, *args, **kwargs: t.Any) -> None:
        import warnings

        warnings.warn('SessionString class is deprecated. Use Session class instead.', DeprecationWarning, stacklevel=2)
        super().__init__(*args, **kwargs)
